#include "/lib/settings.glsl"
#include "/lib/tonemap.glsl"

varying vec3 Normal;
flat varying float material;
varying vec3 ViewPos;
varying float Dist;

#include "/lib/util.glsl"
#include "/global/sky.glsl"
#include "/lib/blur.glsl"

float adjust_torch_falloff(float strength) {
    return strength*strength*1.2;
}
float adjust_sky_falloff(float strength) {
    return max(pow(strength, 1.5), 0);
}

vec3 obscure_moonlight(vec3 LightColor, float NdotL) {
    float obscure_factor = 0;
    if(worldTime > 22000) obscure_factor = cos(PI*(worldTime+2000.)/4000.0);
    else if(worldTime > 13000 && worldTime <= 14000) obscure_factor = sin(PI*(worldTime-1000.0)/2000.0);
    else if(worldTime > 14000) obscure_factor = 1;

    LightColor = apply_saturation(LightColor, 1 - rainStrength) * (1 - rainStrength*0.25);
    return LightColor*NdotL*obscure_factor;
}
// [0] is direct, [1] is ambient
vec3[2] get_sun_color() {
    
    const vec3 SUNRISE_SUN = vec3(SUNRISE_RED, SUNRISE_GREEN, SUNRISE_BLUE);
    const vec3 NOON_SUN = vec3(NOON_RED, NOON_GREEN, NOON_BLUE);
    const vec3 SUNSET_SUN = vec3(SUNSET_RED, SUNSET_GREEN, SUNSET_BLUE);

    vec3[2] FinalSun;
    vec3[2] SkyCol = calc_sky_coef();
    
    #ifdef DIMENSION_NETHER
    FinalSun[1] = vec3(0.55, 0.4, 0.3);
    FinalSun[0] = vec3(0);
    return FinalSun;
    #elif defined DIMENSION_END
    FinalSun[1] = vec3(0.4, 0.3, 0.45);
    FinalSun[0] = vec3(0.2, 0.05, 0.0);
    return FinalSun;
    #endif

    if(worldTime > 23500 || worldTime <= 500) {
        float x = sin(PI*(worldTime+500)/2000.0);
        vec3 amb1 = mix(vec3(0.4), mix(SkyCol[1], SkyCol[0], 0.3), 0.40);
        vec3 amb2 = mix(vec3(0.6), mix(SkyCol[1], SkyCol[0], 0.0), 0.3);
        FinalSun[0] = mix(vec3(0), SUNRISE_SUN, x);
        FinalSun[1] = mix(amb1, amb2, x);
    }
    else if(worldTime <= 1500) {
        float x = sin(PI*(worldTime-500)/2000.0);
        vec3 amb1 = mix(vec3(0.6), mix(SkyCol[1], SkyCol[0], 0.0), 0.3);
        vec3 amb2 = mix(vec3(0.9), mix(SkyCol[1], SkyCol[0], 0.5), 0.4);
        FinalSun[0] = mix(SUNRISE_SUN, NOON_SUN, x);
        FinalSun[1] = mix(amb1, amb2, x);
    }
    else if(worldTime <= 10500) {
        FinalSun[0] = NOON_SUN;
        FinalSun[1] = mix(vec3(0.9), mix(SkyCol[1], SkyCol[0], 0.5), 0.4);
    }
    else if(worldTime <= 12500) {
        float x = sin(PI*(worldTime-2500.0)/4000.0);
        vec3 amb1 = mix(vec3(0.9), mix(SkyCol[1], SkyCol[0], 0.5), 0.4);
        vec3 amb2 = mix(vec3(0.5), mix(SkyCol[1], SkyCol[0], 0.4), 0.35);
        FinalSun[0] = mix(NOON_SUN, SUNSET_SUN, x);
        FinalSun[1] = mix(amb1, amb2, x);
    }
    else if(worldTime <= 13500) {
        float x = sin(PI*(worldTime-500)/2000.0);
        vec3 amb1 = mix(vec3(0.5), mix(SkyCol[1], SkyCol[0], 0.4), 0.35);
        vec3 amb2 = mix(vec3(0.4), mix(SkyCol[1], SkyCol[0], 0.3), 0.40);
        FinalSun[0] = mix(SUNSET_SUN, vec3(0), x);
        FinalSun[1] = mix(amb1, amb2, x);
    }
    else if(worldTime <= 23500) {
        FinalSun[0] = vec3(0);
        FinalSun[1] = mix(vec3(0.4), mix(SkyCol[1], SkyCol[0], 0.3), 0.40);
    }
    FinalSun[0] = apply_saturation(FinalSun[0], 1 - rainStrength/2) * (1 - rainStrength*0.33);
    return FinalSun;
}

vec3 tweak_lightmap(vec2 Lightmap) {
    #ifdef HANDHELD_LIGHTS
    float HandheldLight = max(heldBlockLightValue, heldBlockLightValue2)/15.0;
    Lightmap.x = max(Lightmap.x, HandheldLight-Dist/15.0);
    #endif

    vec2 AdjustedLM = vec2(adjust_torch_falloff(Lightmap.x), adjust_sky_falloff(Lightmap.y));
    vec3 TorchColor = vec3(LM_RED, LM_GREEN, LM_BLUE);
    vec3 MinLight = vec3(MIN_LIGHT_AMOUNT);
     
    #ifndef DIMENSION_OVERWORLD
        AdjustedLM.y = 1;
    #endif

    vec3[2] FinalSun = get_sun_color();
    const vec3 MoonlightColor = vec3(MOON_RED, MOON_GREEN, MOON_BLUE);

    #ifndef DIMENSION_NETHER
    float NdotLSun, NdotLMoon;
    if(material == 10001) {
        NdotLSun = NdotLMoon = 0.4;
    }
    else {
        NdotLSun = max(dot(Normal, normalize(sunPosition)), 0);
        NdotLMoon = max(dot(Normal, normalize(moonPosition)), 0);
    }
    FinalSun[1] += FinalSun[0] * NdotLSun + obscure_moonlight(MoonlightColor, NdotLMoon);
    #endif 

    #ifdef DIMENSION_OVERWORLD
        float TorchNerfFactor = min(get_luminance(FinalSun[0]), 1);

    #elif defined DIMENSION_END
        float TorchNerfFactor = 0.66;
    #else
        float TorchNerfFactor = 0;
    #endif
    AdjustedLM.x = mix(AdjustedLM.x, mix(AdjustedLM.x, 0, TorchNerfFactor), AdjustedLM.y);
    vec3 MixedLights = TorchColor * AdjustedLM.x + FinalSun[1] * AdjustedLM.y;
    MixedLights = mix(MinLight, MixedLights, max(AdjustedLM.x, AdjustedLM.y));
    return dither(MixedLights, gl_FragCoord.xy)*(1-darknessLightFactor);
}
