#include "/lib/noise.glsl"

// [0] is top, [1] is ground
vec3[2] calc_sky_coef() {
    vec3[2] FinalSky;
    const vec3 SKY_TOP_NOON = vec3(0.2, 0.36, 0.7);
    const vec3 SKY_TOP_SUNRISE = vec3(0.1, 0.25, 0.5);
    const vec3 SKY_TOP_SUNSET = vec3(0.07, 0.12, 0.15);
    const vec3 SKY_TOP_NIGHT = vec3(0.02, 0.03, 0.05);

    const vec3 SKY_GROUND_NOON = vec3(0.6, 0.65, 0.9);
    const vec3 SKY_GROUND_SUNRISE = vec3(1.0, 0.55, 0.6);
    const vec3 SKY_GROUND_SUNSET = vec3(1.0, 0.35, 0.2);
    const vec3 SKY_GROUND_NIGHT = vec3(0.15, 0.2, 0.25);

    if(worldTime > 23500 || worldTime <= 500) {
        float x = sin(PI*(worldTime+500)/2000.0);
        FinalSky[0] = mix(SKY_TOP_NIGHT, SKY_TOP_SUNRISE, x);
        FinalSky[1] = mix(SKY_GROUND_NIGHT, SKY_GROUND_SUNRISE, x);
    }
    else if(worldTime <= 1500) {
        float x = sin(PI*(worldTime-500)/2000.0);
        FinalSky[0] = mix(SKY_TOP_SUNRISE, SKY_TOP_NOON, x);
        FinalSky[1] = mix(SKY_GROUND_SUNRISE, SKY_GROUND_NOON, x);
    }
    else if(worldTime <= 10500) {
        FinalSky[0] = SKY_TOP_NOON;
        FinalSky[1] = SKY_GROUND_NOON;
    }
    else if(worldTime <= 12500) {
        float x = sin(PI*(worldTime-2500.0)/4000.0);
        FinalSky[0] = mix(SKY_TOP_NOON, SKY_TOP_SUNSET, x);
        FinalSky[1] = mix(SKY_GROUND_NOON, SKY_GROUND_SUNSET, x);
    }
    else if(worldTime <= 13500) {
        float x = sin(PI*(worldTime-500.0)/2000.0);
        FinalSky[0] = mix(SKY_TOP_SUNSET, SKY_TOP_NIGHT, x);
        FinalSky[1] = mix(SKY_GROUND_SUNSET, SKY_GROUND_NIGHT, x);
    }
    else if(worldTime <= 23500) {
        FinalSky[0] = SKY_TOP_NIGHT;
        FinalSky[1] = SKY_GROUND_NIGHT;
    }
    FinalSky[0] = apply_saturation(FinalSky[0], 1 - rainStrength) * (1 - rainStrength*0.25);
    FinalSky[1] = apply_saturation(FinalSky[1], 1 - rainStrength/2)* (1 - rainStrength*0.33);
    return FinalSky;
} 

float fogify(float x, float w) {
	return w / (x * x + w);
}

vec3 get_clouds(vec3 CloudPos) {
    #if CLOUD_SPEED != 0
        #ifndef DIMENSION_END
        const int ACTUAL_CLOUD_SPEED = 75-CLOUD_SPEED*5;
        float Animation = frameTimeCounter/ACTUAL_CLOUD_SPEED;
        const int ACTUAL_CLOUD_SIZE = CLOUD_SIZE*100;
        #else
        const int ACTUAL_CLOUD_SPEED = 20-CLOUD_SPEED*2;
        float Animation = frameTimeCounter/ACTUAL_CLOUD_SPEED;
        #endif
    #else
        float Animation = 0;
    #endif
	
    float CloudLen = length(CloudPos.xz/CloudPos.y);
    
    #ifndef DIMENSION_END
    float Noise = fbm(CloudPos.xz+Animation, CLOUD_QUALITY);
    Noise = pow(Noise, CLOUD_AMOUNT - rainStrength*6)*ACTUAL_CLOUD_SIZE;

    Noise = ACESFilm(vec3(Noise)).r; // Yes, I'm tonemapping the clouds. Cry about it.
    Noise *= max(1 - CloudLen/24, 0); // Reduce cloud amount further away
    vec3 SkyCol = apply_saturation(calc_sky_coef()[1], 1.2);
	return dither(Noise * SkyCol * (0.7-rainStrength/2), CloudPos.xz);
    #else
    float Noise = fbm(vec2(CloudPos.x-Animation, CloudPos.z+Animation), CLOUD_QUALITY);
    Noise = pow(Noise, 14)*500;

    float BlackHole = max(8-CloudLen, 0);
    if(BlackHole < 0.5) BlackHole *= -BlackHole;
    Noise *= 1 - CloudLen/16;

    return dither((max(Noise, 0)-BlackHole) * vec3(40, 15, 2), CloudPos.xz);
    #endif
}

vec3 calcSkyColor(vec3 pos, vec3 SunVec) {
	float upDot = dot(pos, gbufferModelView[1].xyz)+0.1; //not much, what's up with you?
    vec3[2] FinalSky = calc_sky_coef();

    // Sun glare
    float timeAngle = worldTime / 24000.0;
    float Dist = clamp(dot(pos, SunVec), 0, 1);
    Dist *= Dist;
    float Visibility = cos(timeAngle*2*PI); 
    Visibility *= Visibility;
    Visibility *= Visibility;
    Visibility *= Visibility;
    Visibility *= Visibility;
    
    const vec3 SUN_GLARE = vec3(0.7, 0.25, 0.0);
    FinalSky[1] += SUN_GLARE*Dist*Visibility;

	return dither(mix(FinalSky[0], FinalSky[1], fogify(max(upDot, 0.0), 0.05)), pos.xy);
}
